"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformNumberConstructorCall = exports.transformNumberPrototypeCall = void 0;
const lua = require("../../LuaAST");
const diagnostics_1 = require("../utils/diagnostics");
const lualib_1 = require("../utils/lualib");
const call_1 = require("../visitors/call");
function transformNumberPrototypeCall(context, node) {
    const expression = node.expression;
    const signature = context.checker.getResolvedSignature(node);
    const params = (0, call_1.transformArguments)(context, node.arguments, signature);
    const caller = context.transformExpression(expression.expression);
    const expressionName = expression.name.text;
    switch (expressionName) {
        case "toString":
            return params.length === 0
                ? lua.createCallExpression(lua.createIdentifier("tostring"), [caller], node)
                : (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.NumberToString, node, caller, ...params);
        default:
            context.diagnostics.push((0, diagnostics_1.unsupportedProperty)(expression.name, "number", expressionName));
    }
}
exports.transformNumberPrototypeCall = transformNumberPrototypeCall;
function transformNumberConstructorCall(context, expression) {
    const method = expression.expression;
    const parameters = (0, call_1.transformArguments)(context, expression.arguments);
    const methodName = method.name.text;
    switch (methodName) {
        case "isNaN":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.NumberIsNaN, expression, ...parameters);
        case "isFinite":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.NumberIsFinite, expression, ...parameters);
        default:
            context.diagnostics.push((0, diagnostics_1.unsupportedProperty)(method.name, "Number", methodName));
    }
}
exports.transformNumberConstructorCall = transformNumberConstructorCall;
//# sourceMappingURL=number.js.map